#ifndef _DisplayWindow_h_     // prevent multiple includes
#define _DisplayWindow_h_

#include "../Application.h"
#include "Boundingbox.h"
#include "Camera.h"
#include "Renderer.h"
#include <stdio.h>
#include <stdlib.h>

namespace XMeshLib
{
	class DisplayWindow : public Fl_Gl_Window 
	{
	public:		
		DisplayWindow(int x,int y,int w,int h,const char *l=0);
	
	
		// Overrides	
		void draw();
		int handle(int event);
	
		void SetMesh(XMeshLib::Mesh * inMesh);
		
		void ProcessDraw();
		void ProcessRendering();
		void RenderMesh();

		void RenderFloor();
		void RenderText(const char *text, void *font, int font_flag);
		void RenderTextOnPoint(const char text[], Point & pos);
		void RenderBall(int mat_mode, float size = 1.0f);
		void RenderBall(float R, float G, float B, float size = 1.0f);
		void RenderArrow(float R, float G, float B, float size = 1.0f);
		void RenderCoordinate();
		void RenderBoundingBox();
		void RenderNormal();
		void RenderTessellation();
		void RenderSharpVertex();
		void RenderSharpEdge();
		void RenderSharpFace();
		void RenderMesh4Picking(int pmode);  //To make it compatible with "ProcessHits", primitive names should be their ids


		void ComputeBoundingBox();

	protected:
		XMeshLib::Mesh * theMesh;
		XMeshLib::Boundingbox<double> m_bbox;

	protected:
		void InitSwitches();
		void InitGL();
		void InitCamera();
		void InitLighting();
		void InitMaterial();
		void InitQuadrics();

	protected: //Events Handling

		// (1) Mouse Motions Handling
		void Mousei2f(int x, int y, float *xf, float *yf);
		void MouseDown(int button, int x, int y);
		void MouseUp(int button, int x, int y);
		void MouseMotion(int button, int x, int y);
		void Rotate(float ox, float oy, float nx, float ny);
		void Translate(float dx, float dy, float dz);
		void UpdateDist(float dx, float dy, float dz);


		// (2) Window Reshape		
		void Reshape(GLsizei w, GLsizei h);
		unsigned int Screenw() { return win_width; }
		unsigned int Screenh() { return win_height; }	
		unsigned int GetWinWidth() const { return win_width; }
		unsigned int GetWinHeight() const { return win_height; }
		

		// (3) Picking control functions/variables
		int& PickingStatus()	{ return enable_picking; }
		int& PickingMode()		{ return picking_mode; }
		void ClearPicks();		
		void DoPicking(int x, int y, int pmode);
		void ProcessHits(GLint hits, GLuint buffer[], int pmode);

		std::vector<Face *> pickedFs;
		std::vector<Vertex *> pickedVs;
		std::vector<Edge *> pickedEs;

	//OpenGL environment setting functionals
	public:
		void SetMaterial(int material);
		int& ShadingMode() { return shading_mode; }
		int& PolygonMode() { return polygon_mode; }
		void DoEnableLights();
		void DoDisableLights();

		int& NormalDisplay() { return show_normal; }
		int& TriangulationDisplay() { return show_triangulation; }
		int& SharpVertDisplay() { return show_sharpvert; }
		int& SharpEdgeDisplay() { return show_sharpedge; }
		int& SharpFaceDisplay() { return show_sharpface; }
		int& BoundingBoxDisplay() { return show_boundingbox; }
		int& TexcoordDisplay() { return show_texcoord; }
		int& EnvmapDisplay() { return show_envmap; }
		int& FloorDisplay() { return show_floor; }
		int& StatsDisplay() { return show_stats; }
		void SetBgColor(const GLdouble r, const GLdouble g, const GLdouble b)
		{R = r; G = g; B = b; glClearColor(R, G, B, 0.0f); }
		Camera* GetCamera() { return &theCamera; }


	protected: //Rendering Parameters
		// GL info
		const GLubyte *gl_version;					// OpenGL Version
		const GLubyte *gl_vendor;					// OpenGL Vendor
		const GLubyte *gl_renderer;					// OpenGL PDecompRenderer
		const GLubyte *gl_extensions;				// OpenGL Extensions
		const GLubyte *glu_extensions;				// GLU Extensions
		bool glInitialized;

		// Camera info
		Camera theCamera;							// The single camera used
		float rotate_dist;							// How far the center of the trackball is from the camera
		float view_dist;							// How far the center of the world is from the camera
		// Light info
		bool lights_on[4];							// This light is on ?
		GLfloat light_pos[4][4];					// Position of light source in world coordinate
		
		// Input device info
		int key_down[256];							// Key pressed
		int last_button;							// Last button pressed
		float last_mouse_x, last_mouse_y;			// Last mouse position recorded

		// Window info
		unsigned int win_width, win_height;			// Width and Height of current window
		bool fullscreen;							// Fullscreen rendering
		static GLfloat R, G, B;							// background color

		// Rendering options
		GLuint quadric_list;						// Call-list for quadrics
		GLuint texture2_name[TEX2_COUNT];				// texture names
		

		// Textures variables
		//CTexture m_ctexture2D;							// Texture for surface texture mapping

		int shading_mode;								// Flat/Smooth shading
		int material_mode;								// Current material
		int polygon_mode;								// Point/Line/Fill
		int enable_lighting;							// Enable/disable lighting
		int enable_picking;								// Enable/disable picking
		int picking_mode;								// Picking mode (vertex/edge/face)

		int show_normal;								// Show/hide normal vector
		int show_boundingbox;							// Show/hide boundingbox
		int show_triangulation;							// Show/hide mesh triangulation

		int show_sharpvert;								// Show/hide sharp vertices
		int show_sharpedge;								// Show/hide sharp edges
		int show_sharpface;								// Show/hide sharp faces
		int show_texcoord;								// Show/hide texture coordinate
		int show_envmap;								// Enable/disable environment mapping
		int	show_floor;									// Show/hide floor
		int show_stats;									// Show/hide stats
		
	//====CONSTANT COLORS==========================================================
	private:		
		static const float sharpFaceColor[3]; 
		static const float sharpVerColor[3];
		static const float sharpEdgeColor[3];
		static const float edgeColor[3];
		static const float normalColor[3];	
		static const float surfaceColor[3];
	};
}

#endif


